@@ -21,10 +21,12 @@ class Extension(ext.Extension):
 
     def get_config_schema(self):
         schema = super(Extension, self).get_config_schema()
+        schema['base_dir'] = config.Path(optional=True)
+        schema['default_encoding'] = config.String()
+        schema['default_extension'] = config.String(choices=['.m3u', '.m3u8'])
         schema['playlists_dir'] = config.Path(optional=True)
         return schema
 
     def setup(self, registry):
-        from .actor import M3UBackend
-
+        from .backend import M3UBackend
         registry.add('backend', M3UBackend)
